<?php

	/**
	 * Importer
	 *
	 * @package activeCollab.modules.importer
	 * @subpackage models
	 */
	class Importer {
		
		/**
		 * Mapped companies IDs from /import/companies_map.php file
		 *
		 * @var var
		 */
		protected $companies_map;
		
		/**
		 * Mapped users IDs from /import/users_map.php file
		 *
		 * @var var
		 */
		protected $users_map;
		
		/**
		 * All needed files
		 *
		 * @var array
		 */
		protected $filenames = array('info.php', 'default_categories.php', 'companies.php', 'projects_list.php');
		
		/**
		 * Construct importer
		 *
		 * @param void
		 * @return Importer
		 */
		function __construct() {
			$this->companies_map = Importer::getImportFile('companies_map');
			$this->users_map = Importer::getImportFile('users_map');
		} // __construct
		
		/**
		 * Returns true if there are all files ready to import
		 *
		 * @param void
		 * @return boolean
		 */
		function checkFiles() {
			$files = get_files(IMPORT_PATH);
			if(is_foreachable($files)) {
				foreach($files as $file_path) {
					foreach($this->filenames as $filename) {
						$basename = basename($filename);
						if(!in_array($basename, $this->filenames)) {
							return false;
						} // if
					} // foreach
				} // foreach
			} else {
				return false;
			} // if
			
			return true;
		} // checkFiles
		
		/**
		 * Return import file content
		 *
		 * @param string $file
		 * @return array
		 */
		function getImportFile($file) {
			$import_file = IMPORT_PATH . '/' . $file . '.php';
			if(!is_file($import_file)) {
				return false;
			} // if
			return require($import_file);
		} // getImportFile
		
		/**
		 * Return import project file content
		 *
		 * @param string $file_prefix
		 * @param integer $project_id
		 * @return string
		 */
		function getProjectFile($file_prefix, $project_id) {
			$import_project_file = IMPORT_PATH . '/project_data/' . $file_prefix . '_' . $project_id . '.exp';
			return $import_project_file;
		} // getProjectFile
		
		/**
		 * Return real company ID
		 *
		 * @param integer $company_id
		 * @return integer
		 */
		function getRealCompanyId($company_id) {
			$real_company_id = 0;
			if(is_foreachable($this->companies_map)) {
				foreach($this->companies_map as $id => $real_id) {
					if($id == $company_id) {
						$real_company_id = $real_id;
					} // if
				} // if
			} // if
			return $real_company_id;
		} // getRealCompanyId
		
		/**
		 * Return real user ID
		 *
		 * @param integer $user_id
		 * @return integer
		 */
		function getRealUserId($user_id) {
			$real_user_id = 0;
			
			if(!is_foreachable($this->users_map)) {
				throw new Exception(lang('User map does not exists'));
			} // if
			
			return array_var($this->users_map, $user_id, 0);
		} // getRealUserId
		
		/**
		 * If there are mapped objects, return AC parent ID. Otherwise, return default
		 * value
		 *
		 * @param integer $import_parent_id
		 * @param array $mapped_object_ids
		 * @return integer
		 */
		function getRealParentId($import_parent_id, $mapped_object_ids) {
			$real_parent_id = 0;
			if($import_parent_id != null && is_foreachable($mapped_object_ids)) {
				foreach($mapped_object_ids as $id => $object) {
					if($import_parent_id == $id) {
						$real_parent_id = $object['real_id'];
					} // if
				} // foreach
			} // if
			return $real_parent_id;
		} // getRealParentId
		
		/**
		 * Get current activeCollab master categories
		 *
		 * @param string $module
		 * @return array
		 */
		function getAcCategories($module) {
			// get master categories from activeCollab
			$config_options_table = TABLE_PREFIX . 'config_options';
			$rows = db_execute("SELECT value FROM $config_options_table WHERE name = ?", $module);
			
			$ac_categories = array();
			if(is_foreachable($rows)) {
				foreach($rows as $row) {
					$ac_categories = unserialize($row['value']);
				} // foreach
			} // if
			return $ac_categories;
		} // getAcCategories
		
		/**
		 * Update master categories
		 *
		 * @param array $categories
		 * @param string $name
		 * @return boolean
		 */
		function updateMasterCategories($categories, $name) {
			$config_options_table = TABLE_PREFIX . 'config_options';
			$result = db_execute("UPDATE $config_options_table SET value = ? WHERE name = ?", serialize($categories), $name);
			if(!$result) {
				throw new Error(lang('Query Failed'));
			} // if
			return true;
		} // updateMasterCategories
		
		/**
		 * Update time record parent object
		 *
		 * @param integer $parent_id
		 * @return boolean
		 */
		function updateTimeRecordParentObject($parent_id) {
			$project_objects_table = TABLE_PREFIX . 'project_objects';
			$result = db_execute("UPDATE $project_objects_table SET has_time = ? WHERE id = ?", true, $parent_id);
			if(!$result) {
				throw new Error(lang('Query Failed'));
			} // if
			return true;
		} // updateTimeRecordParentObject
		
		/**
		 * Refresh project progress bar (number of open / total tasks)
		 *
		 * @param integer $project_id
		 * @return boolean
		 */
		function refreshProjectProgressBar($project_id) {
			$count_open_completable = $count_total_completable = 0;
			$completable = get_completable_project_object_types();
			
			if(is_array($completable)) {
				$count_open_completable = ProjectObjects::count(array('project_id = ? AND type IN (?) AND state >= ? AND visibility >= ? AND completed_on IS NULL', $project_id, $completable, STATE_VISIBLE, VISIBILITY_PRIVATE));
				$count_total_completable = ProjectObjects::count(array('project_id = ? AND type IN (?) AND state >= ? AND visibility >= ?', $project_id, $completable, STATE_VISIBLE, VISIBILITY_PRIVATE));
			} // if
    	
    	$projects_table = TABLE_PREFIX . 'projects';
			$result = db_execute("UPDATE $projects_table SET open_tasks_count = ?, total_tasks_count = ? WHERE id = ?", $count_open_completable, $count_total_completable, $project_id);
			if(!$result) {
				throw new Error(lang('Failed to refresh progress bar for project ID: ":id"', array('id' => $project_id)));
			} // if
			
			return true;
		} // refreshProjectProgressBar
		
		/**
		 * Add logged user who importing the data as a project participant (if he's already do not participating in)
		 *
		 * @param User $user
		 * @param integer $project_id
		 * @return boolean
		 */
		function addUserToProject($user, $project_id) {
			$is_already_project_member = (boolean) ProjectUsers::count(array('user_id = ? AND project_id = ?', $user->getId(), $project_id));
			if(!$is_already_project_member) {
				$project_users_table = TABLE_PREFIX . 'project_users';
				
				$result = db_execute("INSERT INTO $project_users_table (user_id, project_id, permissions) VALUES (?, ?, ?)", $user->getId(), $project_id, null);
				if(!$result) {
					throw new Error(lang('Failed to add admin who importing the data as a project user ID: ":id"', array('id' => $user->getId())));
				} // if
				
				clean_user_permissions_cache($user);
      } // if
      
			return true;
		} // addUserToProject
		
		/**
	   * Get file mime type based on a file extension
	   *
	   * @param string $file_path
	   * @return string
	   */
		function get_mime_type($file_path = '') {
			$ct = array(
				'htm' 		=> 'text/html',
	      'html' 		=> 'text/html',
	      'txt' 		=> 'text/plain',
	      'asc' 		=> 'text/plain',
	      'bmp' 		=> 'image/bmp',
	      'gif' 		=> 'image/gif',
	      'jpeg' 		=> 'image/jpeg',
	      'jpg' 		=> 'image/jpeg',
	      'jpe' 		=> 'image/jpeg',
	      'png' 		=> 'image/png',
	      'ico' 		=> 'image/vnd.microsoft.icon',
	      'mpeg' 		=> 'video/mpeg',
	      'mpg' 		=> 'video/mpeg',
	      'mpe' 		=> 'video/mpeg',
	      'qt' 			=> 'video/quicktime',
	      'mov' 		=> 'video/quicktime',
	      'avi' 		=> 'video/x-msvideo',
	      'wmv' 		=> 'video/x-ms-wmv',
	      'mp2' 		=> 'audio/mpeg',
	      'mp3' 		=> 'audio/mpeg',
	      'rm' 			=> 'audio/x-pn-realaudio',
	      'ram' 		=> 'audio/x-pn-realaudio',
	      'rpm' 		=> 'audio/x-pn-realaudio-plugin',
	      'ra' 			=> 'audio/x-realaudio',
	      'wav' 		=> 'audio/x-wav',
	      'css' 		=> 'text/css',
	      'zip' 		=> 'application/zip',
	      'pdf' 		=> 'application/pdf',
	      'doc' 		=> 'application/msword',
	      'bin' 		=> 'application/octet-stream',
	      'exe' 		=> 'application/octet-stream',
	      'class' 	=> 'application/octet-stream',
	      'dll' 		=> 'application/octet-stream',
	      'xls' 		=> 'application/vnd.ms-excel',
	      'ppt' 		=> 'application/vnd.ms-powerpoint',
	      'wbxml' 	=> 'application/vnd.wap.wbxml',
	      'wmlc' 		=> 'application/vnd.wap.wmlc',
	      'wmlsc' 	=> 'application/vnd.wap.wmlscriptc',
	      'dvi' 		=> 'application/x-dvi',
	      'spl' 		=> 'application/x-futuresplash',
	      'gtar' 		=> 'application/x-gtar',
	      'gzip' 		=> 'application/x-gzip',
	      'js' 			=> 'application/x-javascript',
	      'swf' 		=> 'application/x-shockwave-flash',
	      'tar' 		=> 'application/x-tar',
	      'xhtml' 	=> 'application/xhtml+xml',
	      'au' 			=> 'audio/basic',
	      'snd' 		=> 'audio/basic',
	      'midi' 		=> 'audio/midi',
	      'mid' 		=> 'audio/midi',
	      'm3u' 		=> 'audio/x-mpegurl',
	      'tiff' 		=> 'image/tiff',
	      'tif' 		=> 'image/tiff',
	      'rtf' 		=> 'text/rtf',
	      'wml' 		=> 'text/vnd.wap.wml',
	      'wmls' 		=> 'text/vnd.wap.wmlscript',
	      'xsl' 		=> 'text/xml',
	      'xml' 		=> 'text/xml'
			);
	    
	    $extension = get_file_extension($file_path);
	    
	    if(!$mime_type = $ct[strtolower($extension)]) {
	    	$mime_type = 'application/octet-stream';
	    } // if
	    
	    return $mime_type;
	  } // get_mime_type
		
		// ---------------------------------------------------
		//  Company Queries
		// ---------------------------------------------------
		
		/**
		 * Get current activeCollab companies
		 *
		 * @param void
		 * @return array
		 */
		function getAcCompanies() {
			// collect all AC companie names
			$companies_table = TABLE_PREFIX . 'companies';
  		$rows = db_execute("SELECT name FROM $companies_table");
  	
    	$ac_companies = array();
    	if(is_foreachable($rows)) {
	    	foreach($rows as $row) {
	    		$ac_companies[] = $row['name'];
	    	} // foreach
    	} // if
			return $ac_companies;
		} // getAcCompanies
		
		/**
		 * Get owner company real ID
		 *
		 * @param void
		 * @return integer
		 */
		function getOwnerCompanyId() {
			$companies_table = TABLE_PREFIX . 'companies';
			$owner_company_id = db_execute_one("SELECT id FROM $companies_table WHERE is_owner = ?", true);
			return (integer) $owner_company_id['id'];
		} // getOwnerCompanyId
		
		/**
		 * Get company real ID by $name
		 *
		 * @param string $company_name
		 * @return integer
		 */
		function getCompanyIdByName($company_name) {
			$companies_table = TABLE_PREFIX . 'companies';
			$company_id = db_execute_one("SELECT id FROM $companies_table WHERE name = ?", $company_name);
			return (integer) $company_id['id'];
		} // getCompanyIdByName
		
		// ---------------------------------------------------
		//  User Queries
		// ---------------------------------------------------
		
		/**
		 * Get current activeCollab users (their email addresses)
		 *
		 * @param void
		 * @return array
		 */
		function getAcUsers() {
			// collect all email addresses
			$users_table = TABLE_PREFIX . 'users';
    	$rows = db_execute("SELECT email FROM $users_table");
    	
    	$ac_users = array();
    	if(is_foreachable($rows)) {
	    	foreach($rows as $row) {
	    		$ac_users[] = strtolower($row['email']);
	    	} // foreach
    	} // if
			return $ac_users;
		} // getAcUsers
		
		/**
		 * Get user real ID by $email
		 *
		 * @param string $email
		 * @return integer
		 */
		function getUserIdByEmail($email) {
			$users_table = TABLE_PREFIX . 'users';
			$user_id = db_execute_one("SELECT id FROM $users_table WHERE email = ?", $email);
			return (integer) $user_id['id'];
		} // getUserIdByEmail
		
		// ---------------------------------------------------
		//  System
		// ---------------------------------------------------
		
		/**
		 * Prepare and execute an SQL query
		 *
		 * @param string $table_name
		 * @param array $params
		 * @return boolean
		 */
		function prepare_and_execute($table_name, $params) {
			$field_names = array_keys($params);
			$field_values = array_values($params);
						
			$query = "INSERT INTO " . TABLE_PREFIX . $table_name ." (";
			$query.= implode(',', $field_names);
			$query.= ") VALUES (";
			
			$values = array();
			foreach ($field_values as $field_value) {
				$values[] = db_escape($field_value);
			} // foreach
				
			$query.= implode(',', $values);
			$query.= ")";
			
			$result = db_execute($query);
			if(!$result || is_error($result)) {
				if (is_error($result)) {
					throw new Error($result->getMessage());	
				} // if
				throw new Error('Query Failed');
			} // if
			return true;
		} // prepare_and_execute
		
		/**
		 * Update search index
		 *
		 * @param integer $object_id
		 * @param string $type
		 * @param array $searchable_fields
		 * @return boolean
		 */
		function updateSearchIndex($object_id, $type, $searchable_fields) {
			extract($searchable_fields);
			
			$content = '';
			switch($type) {
				case 'User':
					if($content = $email) {
  					if($first_name || $last_name) {
  						$content .= "\n\n" . trim($first_name . ' ' . $last_name);
  					} // if
  				} // if
					break;
				case 'Project':
					if($content = $name) {
						if($overview) {
							$content .= "\n\n" . $overview;
						} // if
					} // if
					break;
				case 'ProjectObject':
					if($name) {
						$content = $name . "\n\n";
					} // if
					if($body) {
						$content .= $body;
					} // if
					break;
			} // switch
			
			if($content) {
				$result = search_index_set($object_id, $type, $content);
				if(!$result || is_error($result)) {
					if(is_error($result)) {
						throw new Exception($result->getMessage());	
					} // if
					throw new Exception('Query Failed');
				} // if
			} // if
			return true;	
		} // updateSearchIndex
		
	}

?>